import serial
import sqlite3
from cobs import cobs
import time
import struct
from dataclasses import dataclass
from datetime import datetime

@dataclass
class Sensor:
	id: int
	accuracy: float
	resolution: float

@dataclass
class DataPoint:
	sensorId: int
	value: float

def ensureDatabaseExists(con: sqlite3.Connection) -> None:
	cur = con.cursor()
	cur.execute("""
		CREATE TABLE IF NOT EXISTS sensor(
			sensorId INTEGER NOT NULL PRIMARY KEY,
			resolution REAL NOT NULL,
			accuracy REAL NOT NULL
		);
	""")

	cur.execute("""
		CREATE TABLE IF NOT EXISTS dataPoint(
			dataPointId INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
			sensorFK INTEGER NOT NULL,
			timestamp INTEGER NOT NULL,
			value REAL NOT NULL,
			FOREIGN KEY(sensorFK) REFERENCES sensor(sensorId)
		);
	""")

def ensureSensorExists(con: sqlite3.Connection, sensor: Sensor) -> None:
	ensureDatabaseExists(con)

	cur = con.cursor()
	res = cur.execute("SELECT COUNT(sensorId) FROM sensor WHERE sensorId = ?;", (sensor.id,))
	if(res.fetchone()[0] == 0):
		cur.execute("INSERT INTO sensor(sensorId, resolution, accuracy) VALUES(?, ?, ?);", (sensor.id, sensor.resolution, sensor.accuracy))

def addDataPoint(con: sqlite3.Connection, dataPoint: DataPoint) -> bool:
	ensureDatabaseExists(con)

	cur = con.cursor()
	res = cur.execute("SELECT COUNT(sensorId) FROM sensor WHERE sensorId = ?;", (dataPoint.sensorId,))
	if(res.fetchone() == 0):
		return False
	cur.execute("INSERT INTO dataPoint(sensorFK, timestamp, value) VALUES(?, ?, ?);", (dataPoint.sensorId, round((datetime.utcnow() - datetime(1970, 1, 1)).total_seconds()), dataPoint.value))

	return True

def read() -> None:
	con = sqlite3.connect("/data/specialIO.db")
	with serial.Serial("/dev/ttyACM0", 9600) as s:
		time.sleep(0.1)
		s.reset_input_buffer()

		while True:
			packet = cobs.decode(s.read_until(expected=b'\x00')[:-1])
			if(packet[0] == 0x01):
				parsed = struct.unpack_from('<HBf', packet, 1)
				sensor = Sensor(parsed[0], parsed[1], parsed[2])
				ensureSensorExists(con, sensor)
				print(f"register: {sensor}")
				con.commit()
			elif(packet[0] == 0x02):
				parsed = struct.unpack_from('<Hf', packet, 1)
				dataPoint = DataPoint(parsed[0], parsed[1])
				addDataPoint(con, dataPoint)
				print(f"data: {dataPoint}")
				con.commit()
			else:
				raise ValueError("Received packet with unknown type", packet)
